using System;
using System.Collections.Generic;
using System.Drawing;
using System.Windows.Forms;

namespace XELF {

	class GraphicsView : GraphicsDeviceControl {
		BoundsMesh BoundsView;
		BoundsMesh WorldBoundsView;
		Trackball trackball;
		Point previous;
		StylusInput stylusInput;
		Point PreviousStylusLocation;
		float StylusPressure;

		protected override void WndProc(ref Message m) {
			if (stylusInput == null || !stylusInput.WndProc(ref m)) { base.WndProc(ref m); }
		}

		public Trackball Trackball { get { return trackball; } }

		public GraphicsView() {
			stylusInput = new StylusInput(Handle);
			stylusInput.Move += new EventHandler(stylusInput_Move);
			stylusInput.PressureChanged += new EventHandler(stylusInput_PressureChanged);
			stylusInput.ButtonDown += new EventHandler(stylusInput_ButtonDown);
			stylusInput.ButtonUp += new EventHandler(stylusInput_ButtonUp);
		}

		void stylusInput_ButtonDown(object sender, EventArgs e) {
			PreviousStylusLocation = stylusInput.Location;
		}
		void stylusInput_ButtonUp(object sender, EventArgs e) {
		}

		void stylusInput_Move(object sender, EventArgs e) {
			if (stylusInput.Pressure >= 0.1f) {
					Trackball.RotateWorld(PreviousStylusLocation, stylusInput.Location);
					BoundsView.World = Trackball.World;
					Invalidate();
			}
			PreviousStylusLocation = stylusInput.Location;
		}
		void stylusInput_PressureChanged(object sender, EventArgs e) {
			StylusPressure = stylusInput.Pressure;
			Trackball.CameraDistance = 20.0f - StylusPressure * 18.0f;
			Invalidate();
		}

		protected override void Initialize() {
			BoundsView = new BoundsMesh(GraphicsDevice);
			WorldBoundsView = new BoundsMesh(GraphicsDevice);
			trackball = new Trackball(GraphicsDevice);
			Trackball.CameraPosition = new Microsoft.Xna.Framework.Vector3(0, 0, -20);
		}

		protected override void OnMouseDown(MouseEventArgs e) {
			previous = e.Location;
		}
		protected override void OnMouseMove(MouseEventArgs e) {
			if ((e.Button & MouseButtons.Left) != 0) {
				Trackball.RotateWorld(previous, e.Location);
				BoundsView.World = Trackball.World;
				Invalidate();
			}
			if ((e.Button & MouseButtons.Right) != 0) {
				Trackball.RotateView(previous, e.Location);
				Invalidate();
			}
			previous = e.Location;
			base.OnMouseMove(e);
		}

		Microsoft.Xna.Framework.Graphics.Color backgroundColor
			= new Microsoft.Xna.Framework .Graphics.Color(0x30, 0x30, 0x30);

		protected override void Draw() {
			GraphicsDevice.Clear(backgroundColor);
			{
				Trackball.View = WorldBoundsView.View = BoundsView.View =
								Microsoft.Xna.Framework.Matrix.CreateLookAt(
								Trackball.CameraPosition,
								Trackball.CameraTarget,
								Microsoft.Xna.Framework.Vector3.Up);

				Trackball.Projection = WorldBoundsView.Projection = BoundsView.Projection
									= Microsoft.Xna.Framework.Matrix.CreatePerspectiveFieldOfView(
									1, GraphicsDevice.Viewport.AspectRatio, 0.01f, 100.0f);

				BoundsView.Update();
				BoundsView.Draw();
				WorldBoundsView.Bounds.Min = new Microsoft.Xna.Framework.Vector3(-3, 0, -3);
				WorldBoundsView.Bounds.Max = new Microsoft.Xna.Framework.Vector3(+3, 0, +3);
				WorldBoundsView.Update();
				WorldBoundsView.Draw();
			}
		}
	}

	class BoundsForm : Form {
		GraphicsView view;
		KeyframePanel keyframePanel;

		protected override void OnLoad(EventArgs e) {
			view.Trackball.WorldChanged += new EventHandler(TrackBall_WorldChanged);
			base.OnLoad(e);
		}

		public BoundsForm() {
			SuspendLayout();
			Text = "BoundsForm";
			ClientSize = new Size(960, 600);

			view = new GraphicsView();
			view.Dock = DockStyle.Fill;
			Controls.Add(view);

			keyframePanel = new KeyframePanel();
			keyframePanel.Dock = DockStyle.Right;
			Controls.Add(keyframePanel);

			ResumeLayout();
		}

		void TrackBall_WorldChanged(object sender, EventArgs e) {
			Microsoft.Xna.Framework.Quaternion rotation = Microsoft.Xna.Framework.Quaternion.CreateFromRotationMatrix(view.Trackball.World);
			keyframePanel.Rotation = rotation;
		}

		void DeviceViewDraw() {
			view.GraphicsDevice.Clear(Microsoft.Xna.Framework.Graphics.Color.DarkBlue);

			view.GraphicsDevice.Present(
				null,
				new Microsoft.Xna.Framework.Rectangle(0, 0, 80 * 3, 60 * 3),
				view.Handle);
		}
	}

}
