using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace XELF {

	class BoundsMesh {
		readonly GraphicsDevice GraphicsDevice;
		VertexDeclaration Declaration;
		VertexBuffer VB;
		IndexBuffer IB;
		Matrix world = Matrix.Identity;
		Matrix deformation = Matrix.Identity;
		BasicEffect Effect;
		public BoundingBox Bounds = new BoundingBox(-Vector3.One, Vector3.One);

		struct Vertex {
			public Vector3 Position;
			public Vertex(Vector3 Position) {
				this.Position = Position;
			}
		}

		public BoundsMesh(GraphicsDevice GraphicsDevice) {
			this.GraphicsDevice = GraphicsDevice;

			Declaration = new VertexDeclaration(GraphicsDevice, new VertexElement[]{
				new VertexElement(0, 0, VertexElementFormat.Vector3, VertexElementMethod.Default, VertexElementUsage.Position, 0),
			});
			VB = new VertexBuffer(GraphicsDevice, typeof(Vertex), 8, BufferUsage.None);
			IB = new IndexBuffer(GraphicsDevice, typeof(int), 36 + 24, BufferUsage.None);

			VB.SetData(new Vertex[]{
				new Vertex(new Vector3(0,0,0)),
				new Vertex(new Vector3(1,0,0)),
				new Vertex(new Vector3(0,1,0)),
				new Vertex(new Vector3(1,1,0)),
				new Vertex(new Vector3(0,0,1)),
				new Vertex(new Vector3(1,0,1)),
				new Vertex(new Vector3(0,1,1)),
				new Vertex(new Vector3(1,1,1)),
			});

			IB.SetData(new int[]{
				// triangle
				0,1,2,3,2,1,
				6,5,4,5,6,7,
				0,2,4,6,4,2,
				3,1,7,5,7,1,
				1,0,5,4,5,0,
				2,3,6,7,6,3,

				// line
				0,1,1,3,3,2,2,0,
				7,6,6,4,4,5,5,7,
				0,4,1,5,2,6,3,7,
			});

			Effect = new BasicEffect(GraphicsDevice, null);
		}

		public void Draw() {
			Effect.DiffuseColor = new Vector3(1, 1, 1);
			Effect.Alpha = 0.25f;
			Effect.Begin();
			Effect.CurrentTechnique.Passes[0].Begin();
			GraphicsDevice.RenderState.AlphaBlendEnable = true;
			GraphicsDevice.RenderState.SourceBlend = Blend.SourceAlpha;
			GraphicsDevice.RenderState.DestinationBlend = Blend.InverseSourceAlpha;
			GraphicsDevice.VertexDeclaration = Declaration;
			GraphicsDevice.Vertices[0].SetSource(VB, 0, Declaration.GetVertexStrideSize(0));
			GraphicsDevice.Indices = IB;
			GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.TriangleList, 0, 0, 8, 0, 12);

			Effect.Alpha = 0.75f;
			Effect.DiffuseColor = new Vector3(1, 0.5f, 0.3f);
			Effect.CommitChanges();
			GraphicsDevice.RenderState.SourceBlend = Blend.One;
			GraphicsDevice.RenderState.DestinationBlend = Blend.One;
			GraphicsDevice.RenderState.DepthBufferEnable = false;
			GraphicsDevice.DrawIndexedPrimitives(PrimitiveType.LineList, 0, 0, 8, 36, 12);
			GraphicsDevice.RenderState.DepthBufferEnable = true;
			Effect.CurrentTechnique.Passes[0].End();
			Effect.End();
		}
		public Matrix World {
			get { return world; }
			set { world = value; }
		}
		public Matrix View {
			get { return Effect.View; }
			set { Effect.View = value; }
		}
		public Matrix Projection {
			get { return Effect.Projection; }
			set { Effect.Projection = value; }
		}

		public void Update() {
			deformation = Matrix.CreateScale(Bounds.Max - Bounds.Min) * Matrix.CreateTranslation(Bounds.Min);
			Effect.World = deformation * world;
		}

	}
}
