#region Using Statements
using System;
using System.Threading;
using Microsoft.Xna.Framework.Graphics;
#endregion

namespace XELF {
	class GraphicsDeviceService : IGraphicsDeviceService {
		static GraphicsDeviceService singletonInstance;
		static int referenceCount;
		const int MaximumSamples = 8;

		GraphicsDevice graphicsDevice;

		PresentationParameters parameters;

		public event EventHandler DeviceCreated;
		public event EventHandler DeviceDisposing;
		public event EventHandler DeviceReset;
		public event EventHandler DeviceResetting;

		GraphicsDeviceService(IntPtr windowHandle, int width, int height) {
			DeviceType deviceType = DeviceType.Hardware;
			GraphicsAdapter adapter = GraphicsAdapter.DefaultAdapter;

			parameters = new PresentationParameters();

			parameters.BackBufferWidth = Math.Max(width, 1);
			parameters.BackBufferHeight = Math.Max(height, 1);
			parameters.BackBufferFormat = SurfaceFormat.Color;
			parameters.PresentationInterval = PresentInterval.Immediate;

			for (int i = MaximumSamples; --i >= 0; ) {
				parameters.MultiSampleType = (MultiSampleType)i;
				if (adapter.CheckDeviceMultiSampleType(
					deviceType,
					parameters.BackBufferFormat,
					parameters.IsFullScreen,
					parameters.MultiSampleType)) break;
			}

			parameters.EnableAutoDepthStencil = true;
			parameters.AutoDepthStencilFormat = DepthFormat.Depth24;

			graphicsDevice = new GraphicsDevice(adapter, deviceType, windowHandle, parameters);
		}

		public static GraphicsDeviceService AddRef(IntPtr windowHandle, int width, int height) {
			if (Interlocked.Increment(ref referenceCount) == 1) {
				singletonInstance = new GraphicsDeviceService(windowHandle, width, height);
			}
			return singletonInstance;
		}

		public void Release(bool disposing) {
			if (Interlocked.Decrement(ref referenceCount) == 0) {
				if (disposing) {
					if (DeviceDisposing != null)
						DeviceDisposing(this, EventArgs.Empty);

					graphicsDevice.Dispose();
				}

				graphicsDevice = null;
			}
		}

		public void ResetDevice(int width, int height) {
			if (DeviceResetting != null)
				DeviceResetting(this, EventArgs.Empty);

			parameters.BackBufferWidth = Math.Max(parameters.BackBufferWidth, width);
			parameters.BackBufferHeight = Math.Max(parameters.BackBufferHeight, height);

			graphicsDevice.Reset(parameters);

			if (DeviceReset != null)
				DeviceReset(this, EventArgs.Empty);
		}

		public GraphicsDevice GraphicsDevice { get { return graphicsDevice; } }
	}
}
