using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;

namespace XELF {

	class Trackball {
		readonly GraphicsDevice GraphicsDevice;
		public Matrix World = Matrix.Identity;
		public Matrix View = Matrix.Identity;
		public Matrix Projection = Matrix.Identity;

		Vector3 cameraPosition;
		public Vector3 CameraPosition {
			get { return cameraPosition; }
			set { cameraPosition = value; }
		}
		public float CameraDistance {
			get {
				return cameraPosition.Length();
			}
			set {
				cameraPosition *= value / cameraPosition.Length();
			}
		}
		public Vector3 CameraTarget;

		public Trackball(GraphicsDevice GraphicsDevice) {
			this.GraphicsDevice = GraphicsDevice;
		}

		public event EventHandler WorldChanged;

		Vector3 ToPosition(System.Drawing.Point location) {
			Vector3 locationOnScreen = new Vector3(location.X, location.Y, 0);
			Vector3 position = GraphicsDevice.Viewport.Unproject(locationOnScreen, Projection, View, Matrix.Identity);
			return position;
		}
		public Quaternion ToRotation(System.Drawing.Point previous, System.Drawing.Point current) {
			Vector3 c = ToPosition(current) - CameraPosition;
			Vector3 p = ToPosition(previous) - CameraPosition;
			Vector3 translation = c - p;
			Vector3 a = translation - CameraPosition;
			c.Normalize();
			p.Normalize();
			if (translation.LengthSquared() < float.Epsilon || a.LengthSquared() < float.Epsilon)
				return Quaternion.Identity;
			translation.Normalize();
			a.Normalize();
			double angle = Math.Acos(MathHelper.Clamp(Vector3.Dot(c, p), -1, +1));
			return Quaternion.CreateFromAxisAngle(Vector3.Cross(translation, a), (float)angle);
		}
		public void RotateWorld(System.Drawing.Point previous, System.Drawing.Point current) {
			Quaternion rotation = ToRotation(previous, current);
			World *= Matrix.CreateFromQuaternion(rotation);
			if (WorldChanged != null) WorldChanged(this, EventArgs.Empty);
		}
		public void RotateView(System.Drawing.Point previous, System.Drawing.Point current) {
			Quaternion rotation = ToRotation(previous, current);
			rotation.Conjugate();
			Matrix transform = Matrix.CreateFromQuaternion(rotation);
			CameraPosition = Vector3.TransformNormal(CameraPosition, transform);
			//if (ViewChanged != null) ViewChanged(this, EventArgs.Empty);
		}
	}

}
